#include "Main.h"
#pragma comment (lib, "shell32")

#include <chrono>
#include <iostream>
#include <fstream>
#include <algorithm>

// Used to open websites in default browser
#include <Windows.h>
#include <shellapi.h>

/// <summary>
/// Rendering
/// </summary>
int Manager::InitRenderer()
{
	// Init video and events
	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_EVENTS) < 0) {
		std::cout << "Error SDL2 Initialization : " << SDL_GetError();
		return 1;
	}

	// Create Window
	m_Window = SDL_CreateWindow("Lego Database",
		SDL_WINDOWPOS_CENTERED_DISPLAY(DISPLAY),
		SDL_WINDOWPOS_CENTERED_DISPLAY(DISPLAY),
		SCRWIDTH, SCRHEIGHT, SDL_WINDOW_BORDERLESS);//SDL_WINDOW_FULLSCREEN_DESKTOP);
	if (!m_Window)
	{
		std::cout << "Failed to create window\n";
		std::cout << "SDL2 Error: " << SDL_GetError() << "\n";
		return 3;
	}

	// Create Surface
	m_WindowSurface = SDL_GetWindowSurface(m_Window);
	if (!m_WindowSurface)
	{
		std::cout << "Failed to get window's surface\n";
		std::cout << "SDL2 Error: " << SDL_GetError() << "\n";
		return 4;
	}

	// Create Renderer
	m_Renderer = SDL_CreateRenderer(m_Window, -1, 0);
	if (!m_Renderer)
	{
		std::cout << "Failed to create renderer\n";
		std::cout << "SDL2 Error: " << SDL_GetError() << "\n";
		return 5;
	}

	// Get ScreenSize
	int w = 0;
	int h = 0;
	SDL_GetWindowSize(m_Window, &w, &h);
	m_ScreenSize = glm::ivec2(w, h);

	return 0;
}

void Manager::CheckEvents()
{
	// Get event
	while (SDL_PollEvent(&m_Event))
	{
		// Check gotten event
		switch (m_Event.type)
		{
		case SDL_QUIT:
			m_ExitApp = true;
			break;
		case SDL_KEYDOWN:
			if (m_Event.key.keysym.sym == SDLK_ESCAPE) {
				m_ExitApp = true;
			}
			else
			{
				// Check if a editable textbox is editing
				bool TextboxEditing = false;
				EditableTextbox* TextBox = nullptr;
				for (UIParent* UI : m_UI)
				{
					TextBox = dynamic_cast<EditableTextbox*>(UI);

					if (TextBox != nullptr && TextBox->Editing)
					{
						TextboxEditing = true;
						break;
					}
				}

				if (TextboxEditing)
				{
					if (m_CurrentMenu == Menus::BrickSelection)
					{
						std::string str = m_BS_PartNum;
						m_BS_PartNum = TextBox->AddCharacter(m_Event.key.keysym.sym);

						if (SearchOnChangeEditableTextbox)	// Separate this, as the compiler otherwise gives warnings
						{
							if (str != m_BS_PartNum)
							{
								m_BS_ShouldReloadBricks = true;
								m_ScreenToLoad = Menus::BrickSelection;
							}
						}
						else if (m_Event.key.keysym.sym == SDLK_RETURN || m_Event.key.keysym.sym == SDLK_KP_ENTER)
						{
							m_BS_ShouldReloadBricks = true;
							m_ScreenToLoad = Menus::BrickSelection;
						}
					}
					else if (m_CurrentMenu == Menus::SetSelection)
					{
						std::string str = m_SS_SetNum;
						m_SS_SetNum = TextBox->AddCharacter(m_Event.key.keysym.sym);

						if (SearchOnChangeEditableTextbox)	// Separate this, as the compiler otherwise gives warnings
						{
							if (str != m_SS_SetNum)
							{
								m_SS_ShouldReloadSets = true;
								m_ScreenToLoad = Menus::SetSelection;
							}
						}
						else if (m_Event.key.keysym.sym == SDLK_RETURN || m_Event.key.keysym.sym == SDLK_KP_ENTER)
						{
							m_SS_ShouldReloadSets = true;
							m_ScreenToLoad = Menus::SetSelection;
						}
					}
				}
			}
			break;
		case SDL_MOUSEMOTION:
			m_MousePos = glm::ivec2(m_Event.motion.x, m_Event.motion.y);
			break;
		case SDL_MOUSEBUTTONUP:
			m_MouseButtonDown = false;
			break;
		case SDL_MOUSEBUTTONDOWN:
			m_MouseButtonDown = true;
			break;
		case SDL_MOUSEWHEEL:
		{
			// Check if a dropdown is open
			bool DropDownOpen = false;
			DropDown* dropdown = nullptr;
			for (UIParent* UI : m_UI)
			{
				dropdown = dynamic_cast<DropDown*>(UI);

				if (dropdown != nullptr && dropdown->Openened)
				{
					DropDownOpen = true;
					break;
				}
			}

			if (DropDownOpen)
				dropdown->Scroll(m_Event.wheel.y);

			break;
		}
		default:
			break;
		}
	}

	m_FrameCount++;
}

void Manager::ClearScreen()
{
	SDL_Rect Rect = SDL_Rect();
	Rect.x = 0;
	Rect.y = 0;
	Rect.w = SCRWIDTH;
	Rect.h = SCRHEIGHT;
	SDL_FillRect(m_WindowSurface, &Rect, SDL_MapRGB(m_WindowSurface->format, m_ClearColor.r, m_ClearColor.g, m_ClearColor.b));
}

void Manager::RenderScreen()
{
	// Make a texture out of the surface, and render the texture onto the renderer
	SDL_Texture* texture = SDL_CreateTextureFromSurface(m_Renderer, m_WindowSurface);
	SDL_RenderCopy(m_Renderer, texture, nullptr, nullptr);

	// Present renderer to screen
	SDL_RenderPresent(m_Renderer);	

	// Destroy texture as otherwise it would create a memory leak
	SDL_DestroyTexture(texture);

	m_ShouldRender = false;
}

void Manager::CapFPS(int FPS)
{
	float time = 1.f / FPS;
	if (m_DeltaTime < time)
	{
		Sleep(DWORD((time - m_DeltaTime) * 1000.f));
		m_DeltaTime = time;
	}
}

/// <summary>
/// UI
/// </summary>
void Manager::UpdateUI()
{
	// Check if someting is opened
	bool SomethingOpen = false;
	DropDown* dropdown = nullptr;
	EditableTextbox* textbox = nullptr;
	for (UIParent* UI : m_UI)
	{
		dropdown = dynamic_cast<DropDown*>(UI);
		textbox = dynamic_cast<EditableTextbox*>(UI);

		if ((dropdown != nullptr && dropdown->Openened) ||
			(textbox != nullptr && textbox->Editing))
		{
			SomethingOpen = true;
			break;
		}
	}

	// Run update function
	if (!SomethingOpen)
	{
		m_EditingTextbox = false;

		for (UIParent* UI : m_UI)
		{
			m_ShouldRender |= UI->Update(m_MousePos, m_MouseButtonDown);
		}
	}
	else
	{
		if (dropdown != nullptr)
			m_ShouldRender |= dropdown->Update(m_MousePos, m_MouseButtonDown);
		else if (textbox != nullptr)
		{
			m_EditingTextbox = true;

			// Calculate if is should print the cursor
			m_DisplayTextboxCursor += m_DeltaTime;
			if (m_DisplayTextboxCursor >= 1.f)
				m_DisplayTextboxCursor -= 1.f;

			textbox->DisplayCursor = m_DisplayTextboxCursor >= 0.5f;
			m_ShouldRender |= textbox->Update(m_MousePos, m_MouseButtonDown);
		}
	}

	// Open menu if needed
	if (m_ScreenToLoad != Menus::NONE)
	{
		RemoveUI();

		switch (m_ScreenToLoad)
		{
		case Menus::LoadImagesScreen: CreateLoadImagesScreen(); break;
		case Menus::StartScreen: CreateStartScreen(); break;
		case Menus::BrickSelection: CreateBrickSelection(); break;
		case Menus::SetSelection: CreateSetSelection(); break;
		case Menus::BrickInfo: CreateBrickInfo(); break;
		case Menus::SetInfo: CreateSetInfo(); break;
		case Menus::ConfirmBrickSelection: CreateConfirmBrickSelection(); break;
		default: printf("Menu not known"); abort();
		}

		m_CurrentMenu = m_ScreenToLoad;
		m_ScreenToLoad = Menus::NONE;
		m_ShouldRender = true;

		// Create reload button if on debug
		if (DEBUG)
		{
			m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
				glm::vec2(0.96f, 0.f), glm::vec2(1.f, 0.05f),
				DEBUGPINK, BLACK,
				"R",
				[this]() {
					m_ScreenToLoad = m_CurrentMenu;
				}));
		}
	}
}

void Manager::RenderUI()
{
	for (UIParent* UI : m_UI)
	{
		UI->Render();
	}
}

void Manager::RemoveUI()
{
	for (UIParent* UI : m_UI)
	{
		delete(UI);
	}

	m_UI.clear();
}

void Manager::CreateLoadImagesScreen()
{
	m_UI.push_back(new Text(m_WindowSurface, m_ScreenSize,
		glm::vec2(0.5f, 0.4f),
		WHITE, true,
		"Do you want to reload the images?"));

	m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
		glm::vec2(0.05f, 0.6f), glm::vec2(0.45f, 0.7f),
		GREY, BLACK,
		"Yes\n(Check console for status)",
		[this]() {
			// Skip the start screen if don't have any bricks
			LoadImages();
			if (m_CustomFilesLoaded)
				m_ScreenToLoad = Menus::StartScreen;
			else
				m_ScreenToLoad = Menus::BrickSelection;
		}));

	m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
		glm::vec2(0.55f, 0.6f), glm::vec2(0.95f, 0.7f),
		GREY, BLACK,
		"No",
		[this]() {
			// Skip the start screen if don't have any bricks
			if (m_CustomFilesLoaded)
				m_ScreenToLoad = Menus::StartScreen;
			else
				m_ScreenToLoad = Menus::BrickSelection;
		}));
}

void Manager::CreateStartScreen()
{
	m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
		glm::vec2(0.05f, 0.6f), glm::vec2(0.45f, 0.7f),
		GREY, BLACK,
		"See possible sets",
		[this]() {
			CheckPossibleSets();
			m_ScreenToLoad = Menus::SetSelection;
		}));

	m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
		glm::vec2(0.55f, 0.6f), glm::vec2(0.95f, 0.7f),
		GREY, BLACK,
		"Continue with saved bricks",
		[this]() {
			m_ScreenToLoad = Menus::BrickSelection;
		}));

	m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
		glm::vec2(0.3f, 0.71f), glm::vec2(0.7f, 0.81f),
		GREY, BLACK,
		"Start from scratch",
		[this]() {
			// Reset bricks
			for (GottenBricks& Brick : m_AllBricks)
			{
				Brick.Quantity = 0;
			}

			// Reset sets
			for (Sets& Set : m_Sets)
			{
				Set.Quantity = 0;
			}

			m_ScreenToLoad = Menus::BrickSelection;
		}));

	m_UI.push_back(new Text(m_WindowSurface, m_ScreenSize,
		glm::vec2(0.5f, 0.4f),
		WHITE, true,
		"Last time you have saved some bricks.\n\n\n"
		"Choose one of the following options:"));
}

void Manager::CreateBrickSelection()
{
	const float rows = 10;
	const float columns = 5;

	// Calculate the bricks to display
	if (m_BS_ShouldReloadBricks)
	{
		m_BS_ShouldReloadBricks = false;
		m_BS_DisplayBricks.clear();

		m_BS_DisplayBricks = GetDisplayableBricks(m_AllBricks, true, true, false);

		// Clamp page to the possible page numbers
		int MaxPage = (int)ceil((float)m_BS_DisplayBricks.size() / (rows * columns));
		m_BS_Page = CLAMP(m_BS_Page, 1, MaxPage);
	}

	// Apply button
	{
		m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
			glm::vec2(0.8f, 0.93f), glm::vec2(0.99f, 0.98f),
			GREY, BLACK,
			"Apply",
			[this]() {
				m_ScreenToLoad = Menus::ConfirmBrickSelection;
				m_BS_Page = 1;
			}));
	}

	// Show all/selection button
	{
		m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
			glm::vec2(0.01f, 0.93f), glm::vec2(0.2f, 0.98f),
			GREY, BLACK,
			m_ShowSelectedOnly ? "Show all" : "Show selection",
			[this]() {
				m_ShowSelectedOnly = !m_ShowSelectedOnly;
				m_ScreenToLoad = Menus::BrickSelection;
				m_BS_ShouldReloadBricks = true;
			}));
	}

	// Switch to sets
	{
		m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
			glm::vec2(0.01f, 0.02f), glm::vec2(0.2f, 0.08f),
			GREY, BLACK,
			"Show sets",
			[this]() {
				m_ScreenToLoad = Menus::SetSelection;
				m_SS_ShouldReloadSets = true;
				m_SS_ComingFromBS = true;
			}));
	}

	// Page selector
	{
		// 1 page
		{
			if (m_BS_Page > 1)
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.4f, 0.93f), glm::vec2(0.45f, 0.98f),
					GREY, BLACK,
					"<",
					[this]() {
						m_BS_Page--;
				m_ScreenToLoad = Menus::BrickSelection;
					}));
			}

			if ((int)m_BS_DisplayBricks.size() > ((int)rows * (int)columns * m_BS_Page))
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.55f, 0.93f), glm::vec2(0.6f, 0.98f),
					GREY, BLACK,
					">",
					[this]() {
						m_BS_Page++;
				m_ScreenToLoad = Menus::BrickSelection;
					}));
			}
		}

		// 10 page
		{
			if (m_BS_Page > 10)
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.345f, 0.93f), glm::vec2(0.395f, 0.98f),
					GREY, BLACK,
					"<<",
					[this]() {
						m_BS_Page -= 10;
						m_ScreenToLoad = Menus::BrickSelection;
					}));
			}

			if ((int)m_BS_DisplayBricks.size() > ((int)rows * (int)columns * (m_BS_Page + 9)))
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.605f, 0.93f), glm::vec2(0.655f, 0.98f),
					GREY, BLACK,
					">>",
					[this]() {
						m_BS_Page += 10;
						m_ScreenToLoad = Menus::BrickSelection;
					}));
			}
		}

		// Page counter
		m_UI.push_back(new Text(m_WindowSurface, m_ScreenSize,
			glm::vec2(0.5f, 0.95f),
			WHITE, true,
			std::to_string(m_BS_Page)));
	}

	// Create tiles
	{
		if (m_BS_DisplayBricks.size() == 0)
		{
			m_UI.push_back(new Text(m_WindowSurface, m_ScreenSize,
				glm::vec2(0.5f, 0.5f),
				WHITE, true,
				"No results found"));
		}
		else
		{
			const float left_margin = 0.01f;
			const float right_margin = 0.995f;
			const float top_margin = 0.1f;
			const float bottom_margin = 0.92f;

			for (int y = 0; y < columns; y++)
			{
				for (int x = 0; x < rows; x++)
				{
					int index = x + (y * (int)rows) + ((m_BS_Page - 1) * (int)rows * (int)columns);
					if (index < (int)m_BS_DisplayBricks.size())
					{
						GottenBricks* item = &m_BS_DisplayBricks[index];

						// Create actual tile
						glm::vec2 TL = glm::vec2(
							left_margin + ((float)x / rows) * (right_margin - left_margin),
							top_margin + ((float)y / columns) * (bottom_margin - top_margin));

						glm::vec2 BR = glm::vec2(
							left_margin + (((float)x + 0.95f) / rows) * (right_margin - left_margin),
							top_margin + (((float)y + 0.95f) / columns) * (bottom_margin - top_margin));

						m_UI.push_back(new Tile(m_WindowSurface, m_ScreenSize,
							TL, BR,
							GREY, BLACK,
							item, true, &m_InventoryParts,
							[this](GottenBricks* brick) {
								m_BI_SelectedBrick = brick;
								m_ScreenToLoad = Menus::BrickInfo;
							}));
					}
				}
			}
		}
	}

	// Create advanced searching options
	{
		// Colors dropdown
		{
			// Get vector
			std::vector<GottenBricks> UseableBricks = GetDisplayableBricks(m_AllBricks, false, true, false);

			// Get count
			std::vector<int> ColorCount((m_Colors.size()));
			for (GottenBricks Brick : UseableBricks)
			{
				std::vector<Colors>::iterator ColorIt = std::find(m_Colors.begin(), m_Colors.end(), Brick.ColorID);
				if (ColorIt != m_Colors.end())
				{
					int index = ColorIt - m_Colors.begin();
					ColorCount[index]++;
				}
			}

			// Create dropdown items
			std::vector<DropDownItem> Items;
			Items.push_back(DropDownItem("Reset Filter", UseableBricks.size()));
			int i = 0;
			for (int amount : ColorCount)
			{
				std::string name = m_Colors[i].Name;
				if (name == "[Unknown]")
					name = "Unknown";
				else if (name == "[No Color/Any Color]")
					name = "No Color";

				Items.push_back(DropDownItem(name, amount));
				i++;
			}

			// Get current selected item
			int Selected = -2;
			std::vector<Colors>::iterator SelectedIt = std::find(m_Colors.begin(), m_Colors.end(), m_BS_Color);
			if (SelectedIt != m_Colors.end())
			{
				Selected = SelectedIt - m_Colors.begin();
			}

			// Create dropdown
			m_UI.push_back(new DropDown(m_WindowSurface, m_ScreenSize,
				glm::vec2(0.39f, 0.02f), glm::vec2(0.77f, 0.08f),
				GREY, BLACK,
				"Color",
				Items,
				Selected,
				[this](int item) {
					if (item == -1)
						m_BS_Color = -2;
					else
						m_BS_Color = m_Colors[item].ColorID;

					m_BS_ShouldReloadBricks = true;
					m_ScreenToLoad = Menus::BrickSelection;
				}));
		}

		// PartNum editable text box
		{
			m_UI.push_back(new EditableTextbox(m_WindowSurface, m_ScreenSize,
				glm::vec2(0.78f, 0.02f), glm::vec2(0.99f, 0.08f),
				GREY, DARKGREY, BLACK,
				"Search part number", m_BS_PartNum,
				m_EditingTextbox, 20));
		}
	}
}

void Manager::CreateSetSelection()
{
	const float rows = 10;
	const float columns = 5;

	// Calculate the sets to display
	if (m_SS_ShouldReloadSets)
	{
		m_SS_ShouldReloadSets = false;
		m_SS_DisplaySets.clear();

		if (m_SS_ComingFromBS)
			m_SS_DisplaySets = GetDisplayableSets(m_Sets, true, false, false);
		else
			m_SS_DisplaySets = GetDisplayableSets(m_PossibleSets, true, false, !m_SS_ShowMinifigSets);

		// Clamp page to the possible page numbers
		int MaxPage = (int)ceil((float)m_SS_DisplaySets.size() / (rows * columns));
		m_SS_Page = CLAMP(m_SS_Page, 1, MaxPage);
	}

	// Create brick selection UI
	{
		if (m_SS_ComingFromBS)
		{
			// Switch to bricks
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.01f, 0.02f), glm::vec2(0.2f, 0.08f),
					GREY, BLACK,
					"Show bricks",
					[this]() {
						m_ScreenToLoad = Menus::BrickSelection;
				m_BS_ShouldReloadBricks = true;
					}));
			}

			// Show all/selection button
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.01f, 0.93f), glm::vec2(0.2f, 0.98f),
					GREY, BLACK,
					m_ShowSelectedOnly ? "Show all" : "Show selection",
					[this]() {
						m_ShowSelectedOnly = !m_ShowSelectedOnly;
				m_ScreenToLoad = Menus::SetSelection;
				m_SS_ShouldReloadSets = true;
					}));
			}

			// Apply button
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.8f, 0.93f), glm::vec2(0.99f, 0.98f),
					GREY, BLACK,
					"Apply",
					[this]() {
						m_ScreenToLoad = Menus::ConfirmBrickSelection;
				m_BS_Page = 1;
					}));
			}
		}
		else
		{
			// Show minifig sets button
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.01f, 0.93f), glm::vec2(0.3f, 0.98f),
					GREY, BLACK,
					m_SS_ShowMinifigSets ? "Show normal sets" : "Show all sets",
					[this]() {
						m_SS_ShowMinifigSets = !m_SS_ShowMinifigSets;

						m_ScreenToLoad = Menus::SetSelection;
						m_SS_ShouldReloadSets = true;
					}));
			}

			// Text
			{
				m_UI.push_back(new Text(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.5f, 0.05f),
					WHITE, true,
					"You can make these sets"));
			}
		}
	}

	// Page selector
	{
		// 1 page
		{
			if (m_SS_Page > 1)
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.4f, 0.93f), glm::vec2(0.45f, 0.98f),
					GREY, BLACK,
					"<",
					[this]() {
						m_SS_Page--;
						m_ScreenToLoad = Menus::SetSelection;
					}));
			}

			if ((int)m_SS_DisplaySets.size() > ((int)rows * (int)columns * m_SS_Page))
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.55f, 0.93f), glm::vec2(0.6f, 0.98f),
					GREY, BLACK,
					">",
					[this]() {
						m_SS_Page++;
						m_ScreenToLoad = Menus::SetSelection;
					}));
			}
		}

		// 10 page
		{
			if (m_SS_Page > 10)
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.345f, 0.93f), glm::vec2(0.395f, 0.98f),
					GREY, BLACK,
					"<<",
					[this]() {
						m_SS_Page -= 10;
						m_ScreenToLoad = Menus::SetSelection;
					}));
			}

			if ((int)m_SS_DisplaySets.size() > ((int)rows * (int)columns * (m_SS_Page + 9)))
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.605f, 0.93f), glm::vec2(0.655f, 0.98f),
					GREY, BLACK,
					">>",
					[this]() {
						m_SS_Page += 10;
						m_ScreenToLoad = Menus::SetSelection;
					}));
			}
		}

		// Page counter
		m_UI.push_back(new Text(m_WindowSurface, m_ScreenSize,
			glm::vec2(0.5f, 0.95f),
			WHITE, true,
			std::to_string(m_SS_Page)));
	}

	// Create tiles
	{
		if (m_SS_DisplaySets.size() == 0)
		{
			m_UI.push_back(new Text(m_WindowSurface, m_ScreenSize,
				glm::vec2(0.5f, 0.5f),
				WHITE, true,
				"No results found"));
		}
		else
		{
			const float left_margin = 0.01f;
			const float right_margin = 0.995f;
			const float top_margin = 0.1f;
			const float bottom_margin = 0.92f;

			for (int y = 0; y < columns; y++)
			{
				for (int x = 0; x < rows; x++)
				{
					int index = x + (y * (int)rows) + ((m_SS_Page - 1) * (int)rows * (int)columns);
					if (index < (int)m_SS_DisplaySets.size())
					{
						Sets* item = &m_SS_DisplaySets[index];

						// Create actual tile
						glm::vec2 TL = glm::vec2(
							left_margin + ((float)x / rows) * (right_margin - left_margin),
							top_margin + ((float)y / columns) * (bottom_margin - top_margin));

						glm::vec2 BR = glm::vec2(
							left_margin + (((float)x + 0.95f) / rows) * (right_margin - left_margin),
							top_margin + (((float)y + 0.95f) / columns) * (bottom_margin - top_margin));

						m_UI.push_back(new Tile(m_WindowSurface, m_ScreenSize,
							TL, BR,
							GREY, BLACK,
							item, m_SS_ComingFromBS,
							[this](Sets* set) {
								// Switch menus
								m_SI_SelectedSet = set;
						m_ScreenToLoad = Menus::SetInfo;
							}));
					}
				}
			}
		}
	}

	// Create advanced searching options
	{
		// SetNum editable text box
		{
			m_UI.push_back(new EditableTextbox(m_WindowSurface, m_ScreenSize,
				glm::vec2(0.78f, 0.02f), glm::vec2(0.99f, 0.08f),
				GREY, DARKGREY, BLACK,
				"Search set number", m_SS_SetNum,
				m_EditingTextbox, 20));
		}
	}
}

void Manager::CreateBrickInfo()
{
	// Check if brick is selected
	if (m_BI_SelectedBrick == nullptr)
	{
		printf("No valid brick selected\n");
		abort();
	}

	// Image
	{
		// Get URL
		std::string URL;
		std::vector<InventoryParts>::iterator IPIt = std::find(m_InventoryParts.begin(), m_InventoryParts.end(), *m_BI_SelectedBrick);
		if (IPIt != m_InventoryParts.end())
		{
			int index = IPIt - m_InventoryParts.begin();
			URL = m_InventoryParts.at(index).ImgURL;
		}

		std::string ImagePath = "Files/Images/NotFound.png";
		if (!URL.empty())
		{
			// Get Path
			std::string Type = URL.substr(URL.length() - 4);
			ImagePath = "Files/Images/Parts/" + m_BI_SelectedBrick->PartNum + "_" + std::to_string(m_BI_SelectedBrick->ColorID) + Type;
		}

		// Create Image
		m_UI.push_back(new Image(m_WindowSurface, m_ScreenSize,
			glm::vec2(0.3f, 0.02f), glm::vec2(0.7f, 0.7f),
			URL, ImagePath));
	}

	// Apply button
	{
		m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
			glm::vec2(0.8f, 0.93f), glm::vec2(0.99f, 0.98f),
			GREY, BLACK,
			"Apply",
			[this]() {
				// Set quantity for brick
				std::vector<GottenBricks>::iterator BrickIt = std::find(m_AllBricks.begin(), m_AllBricks.end(), *m_BI_SelectedBrick);
				if (BrickIt == m_AllBricks.end())
				{
					printf("Brick not found");
					abort();
				}
				int index = BrickIt - m_AllBricks.begin();
				m_AllBricks[index] = *m_BI_SelectedBrick;

				// Load menu
				m_ScreenToLoad = Menus::BrickSelection;
				m_BS_ShouldReloadBricks = true;
			}));
	}

	// Cancel button
	{
		m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
			glm::vec2(0.01f, 0.93f), glm::vec2(0.2f, 0.98f),
			GREY, BLACK,
			"Cancel",
			[this]() {
				m_ScreenToLoad = Menus::BrickSelection;
				m_BS_ShouldReloadBricks = true;
			}));
	}

	// Count selector
	{
		// 1
		{
			if (m_BI_SelectedBrick->Quantity >= 1)
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.4f, 0.93f), glm::vec2(0.45f, 0.98f),
					GREY, BLACK,
					"-",
					[this]() {
						m_BI_SelectedBrick->Quantity--;
				m_ScreenToLoad = Menus::BrickInfo;
					}));
			}

			m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
				glm::vec2(0.55f, 0.93f), glm::vec2(0.6f, 0.98f),
				GREY, BLACK,
				"+",
				[this]() {
					m_BI_SelectedBrick->Quantity++;
					m_ScreenToLoad = Menus::BrickInfo;
				}));
		}

		// 10
		{
			if (m_BI_SelectedBrick->Quantity >= 10)
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.345f, 0.93f), glm::vec2(0.395f, 0.98f),
					GREY, BLACK,
					"--",
					[this]() {
						m_BI_SelectedBrick->Quantity -= 10;
						m_ScreenToLoad = Menus::BrickInfo;
					}));
			}

			m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
				glm::vec2(0.605f, 0.93f), glm::vec2(0.655f, 0.98f),
				GREY, BLACK,
				"++",
				[this]() {
					m_BI_SelectedBrick->Quantity += 10;
					m_ScreenToLoad = Menus::BrickInfo;
				}));
		}

		// count counter
		m_UI.push_back(new Text(m_WindowSurface, m_ScreenSize,
			glm::vec2(0.5f, 0.95f),
			WHITE, true,
			std::to_string(m_BI_SelectedBrick->Quantity)));
	}

	// Brick info
	{
		// Get Part
		Parts* Part = nullptr;
		{
			std::vector<Parts>::iterator PartIt = std::find(m_Parts.begin(), m_Parts.end(), m_BI_SelectedBrick->PartNum);
			if (PartIt == m_Parts.end())
			{
				printf("Part not found, PartNum: %s", m_BI_SelectedBrick->PartNum.c_str());
				abort();
			}
			else
			{
				int PartIndex = PartIt - m_Parts.begin();
				Part = &m_Parts[PartIndex];
			}
		}

		// Get Color
		std::string Color = "";
		{
			std::vector<Colors>::iterator ColorIt = std::find(m_Colors.begin(), m_Colors.end(), m_BI_SelectedBrick->ColorID);
			if (ColorIt == m_Colors.end())
			{
				printf("Color not found, Color: %i", m_BI_SelectedBrick->ColorID);
				abort();
			}
			else
			{
				int ColorIndex = ColorIt - m_Colors.begin();
				Color = m_Colors[ColorIndex].Name;
			}
		}

		// Get Category
		std::string Category = "";
		{
			std::vector<PartCategories>::iterator CategoryIt = std::find(m_PartCategories.begin(), m_PartCategories.end(), Part->PartCatID);
			if (CategoryIt == m_PartCategories.end())
			{
				printf("Part category not found, Category: %i", Part->PartCatID);
				abort();
			}
			else
			{
				int CategoryIndex = CategoryIt - m_PartCategories.begin();
				Category = m_PartCategories[CategoryIndex].Name;
			}
		}

		// Get PartRelationship
		PartRelationships* PartRelationship = nullptr;
		{
			std::vector<PartRelationships>::iterator PartRelationshipIt = std::find(m_PartRelationships.begin(), m_PartRelationships.end(), Part->PartNum);			
			if (PartRelationshipIt != m_PartRelationships.end())	// Relationship doesn't have to exist, part could also have no parents
			{
				int PartRelationshipIndex = PartRelationshipIt - m_PartRelationships.begin();
				PartRelationship = &m_PartRelationships[PartRelationshipIndex];
			}
		}

		// Get text
		std::string text = Part->Name + "\n" +
			Part->PartNum + "\n" +
			"Color: " + Color + "\n" +
			"Material: " + Part->PartMaterial + "\n" +
			"Category: " + Category + "\n";
		if (PartRelationship != nullptr)
		{
			text += PartRelationship->RelativeType + " of part: " + PartRelationship->ParentPartNum;
		}

		// Create text
		m_UI.push_back(new Text(m_WindowSurface, m_ScreenSize,
			glm::vec2(0.5f, 0.79f),
			WHITE, true,
			text));
	}
}

void Manager::CreateSetInfo()
{
	// Check if set is selected
	if (m_SI_SelectedSet == nullptr)
	{
		printf("No valid set selected\n");
		abort();
	}

	// Image
	{
		// Get URL
		std::string URL = m_SI_SelectedSet->ImgURL;

		std::string ImagePath = "Files/Images/NotFound.png";
		if (!URL.empty())
		{
			// Get Path
			std::string Type = URL.substr(URL.length() - 4);
			ImagePath = "Files/Images/Sets/" + m_SI_SelectedSet->SetNum + Type;
		}

		// Create Image
		m_UI.push_back(new Image(m_WindowSurface, m_ScreenSize,
			glm::vec2(0.3f, 0.02f), glm::vec2(0.7f, 0.7f),
			URL, ImagePath));
	}

	// Create brick selection UI
	{
		if (m_SS_ComingFromBS)
		{
			// Apply button
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.8f, 0.93f), glm::vec2(0.99f, 0.98f),
					GREY, BLACK,
					"Apply",
					[this]() {
						// Get index for set
						std::vector<Sets>::iterator SetIt = std::find(m_Sets.begin(), m_Sets.end(), *m_SI_SelectedSet);
						if (SetIt == m_Sets.end())
						{
							printf("Set not found");
							abort();
						}
						int SetIndex = SetIt - m_Sets.begin();

						// Add/Remove bricks according to this set
						int diff = m_SI_SelectedSet->Quantity - m_Sets[SetIndex].Quantity;
						if (diff != 0)
						{
							// Get InventoryID
							std::vector<Inventories>::iterator InventoryIt = std::find(m_Inventories.begin(), m_Inventories.end(), m_SI_SelectedSet->SetNum);
							if (InventoryIt != m_Inventories.end())
							{
								int InventoryIndex = InventoryIt - m_Inventories.begin();

								// Get corresponding InventoryParts
								std::pair<std::vector<InventoryParts>::iterator, std::vector<InventoryParts>::iterator> bounds;
								bounds = std::equal_range(m_InventoryParts.begin(), m_InventoryParts.end(), m_Inventories[InventoryIndex].ID);

								int low = bounds.first - m_InventoryParts.begin();
								int up = bounds.second - m_InventoryParts.begin();
								for (int i = low; i < up; i++)
								{
									// Get Element
									Elements Element = Elements(m_InventoryParts[i].PartNum, m_InventoryParts[i].ColorID);

									// Get Gottenbrick index
									std::vector<GottenBricks>::iterator GottenBricksIt = std::find(m_AllBricks.begin(), m_AllBricks.end(), Element);
									if (GottenBricksIt == m_AllBricks.end())
									{
										// Brick isn't in m_AllBricks, maybe try running on Release?
										printf("Brick: %s not recognized\n", m_InventoryParts[i].PartNum.c_str());
										abort();
									}
									else
									{
										int GottenBricksIndex = GottenBricksIt - m_AllBricks.begin();

										// Add/Remove bricks
										m_AllBricks[GottenBricksIndex].Quantity = MAX(0, m_AllBricks[GottenBricksIndex].Quantity + (m_InventoryParts[i].Quantity * diff));
									}									
								}
							}
						}

						// Load menu
						m_Sets[SetIndex] = *m_SI_SelectedSet;
						m_ScreenToLoad = Menus::SetSelection;
						m_SS_ShouldReloadSets = true;
					}));
			}

			// Count selector
			{
				// 1
				{
					if (m_SI_SelectedSet->Quantity >= 1)
					{
						m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
							glm::vec2(0.4f, 0.93f), glm::vec2(0.45f, 0.98f),
							GREY, BLACK,
							"-",
							[this]() {
								m_SI_SelectedSet->Quantity--;
						m_ScreenToLoad = Menus::SetInfo;
							}));
					}

					m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
						glm::vec2(0.55f, 0.93f), glm::vec2(0.6f, 0.98f),
						GREY, BLACK,
						"+",
						[this]() {
							m_SI_SelectedSet->Quantity++;
					m_ScreenToLoad = Menus::SetInfo;
						}));
				}

				// 10
				{
					if (m_SI_SelectedSet->Quantity >= 10)
					{
						m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
							glm::vec2(0.345f, 0.93f), glm::vec2(0.395f, 0.98f),
							GREY, BLACK,
							"--",
							[this]() {
								m_SI_SelectedSet->Quantity -= 10;
						m_ScreenToLoad = Menus::SetInfo;
							}));
					}

					m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
						glm::vec2(0.605f, 0.93f), glm::vec2(0.655f, 0.98f),
						GREY, BLACK,
						"++",
						[this]() {
							m_SI_SelectedSet->Quantity += 10;
					m_ScreenToLoad = Menus::SetInfo;
						}));
				}

				// count counter
				m_UI.push_back(new Text(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.5f, 0.95f),
					WHITE, true,
					std::to_string(m_SI_SelectedSet->Quantity)));
			}
		}
		else
		{
			// Building instructions
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.7f, 0.93f), glm::vec2(0.99f, 0.98f),
					GREY, BLACK,
					"Building Instructions",
					[this]() {
						std::string URL = "https://www.lego.com/en-us/service/buildinginstructions/" + m_SI_SelectedSet->SetNum.substr(0, m_SI_SelectedSet->SetNum.length() - 2);

						ShellExecute(NULL, NULL,
						std::wstring(URL.begin(), URL.end()).c_str(),
						NULL, NULL, SW_SHOWNORMAL);
					}));
			}

			// Ignore Set
			{
				std::vector<std::string>::iterator StrIt = std::find(m_MinifigSets.begin(), m_MinifigSets.end(), m_SI_SelectedSet->SetNum);
				if (StrIt == m_MinifigSets.end())
				{
					// Create button
					m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
						glm::vec2(0.85f, 0.01f), glm::vec2(0.99f, 0.06f),
						GREY, BLACK,
						"Ignore Set",
						[this]() {
							// Add to current list
							m_MinifigSets.push_back(m_SI_SelectedSet->SetNum);

							// Add set to ignore list
							FILE* stream;
							fopen_s(&stream, "Files/CSVs/other_sets.csv", "a");
							if (!stream)
							{
								printf("File other_sets.csv wasn't created!");
								abort();
							}

							fprintf(stream, (m_SI_SelectedSet->SetNum + std::string("\n")).c_str());

							fclose(stream);
							printf("Set %s successfully saved.\n", m_SI_SelectedSet->SetNum.c_str());

							// Reload menu
							m_ScreenToLoad = Menus::SetInfo;
						}));
				}
				else
				{
					// Create text
					m_UI.push_back(new Text(m_WindowSurface, m_ScreenSize,
						glm::vec2(0.85f, 0.02f),
						WHITE, false,
						"Set Ignored"));
				}
			}
		}
	}
	
	// Cancel/Back button
	{
		m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
			glm::vec2(0.01f, 0.93f), glm::vec2(0.2f, 0.98f),
			GREY, BLACK,
			m_SS_ComingFromBS ? "Cancel" : "Back",
			[this]() {
				m_ScreenToLoad = Menus::SetSelection;
				m_SS_ShouldReloadSets = true;
			}));
	}

	// Set info
	{
		// Get theme
		std::string SubTheme = "";
		std::string Theme = "";
		{
			std::vector<Themes>::iterator SubThemeIt = std::find(m_Themes.begin(), m_Themes.end(), m_SI_SelectedSet->ThemeID);
			if (SubThemeIt == m_Themes.end())
			{
				printf("Theme not found, ID: %i", m_SI_SelectedSet->ThemeID);
				abort();
			}
			else
			{
				int SubThemeIndex = SubThemeIt - m_Themes.begin();
				SubTheme = m_Themes[SubThemeIndex].Name;

				// Search for a possible subtheme
				if (m_Themes[SubThemeIndex].ParentID != -1)
				{
					std::vector<Themes>::iterator ThemeIt = std::find(m_Themes.begin(), m_Themes.end(), m_Themes[SubThemeIndex].ParentID);
					if (ThemeIt == m_Themes.end())
					{
						printf("Theme not found, ID: %i", m_Themes[SubThemeIndex].ParentID);
						abort();
					}
					else
					{
						int ThemeIndex = ThemeIt - m_Themes.begin();
						Theme = m_Themes[ThemeIndex].Name;
					}
				}
			}
		}

		// Get text
		std::string text = m_SI_SelectedSet->Name + "\n" +
			m_SI_SelectedSet->SetNum + "\n" +
			"Parts: " + std::to_string(m_SI_SelectedSet->NumParts) + "\n" +
			"Year of release: " + std::to_string(m_SI_SelectedSet->Year) + "\n";
		if (Theme.empty())
		{
			text += "Theme: " + SubTheme + "\n";
		}
		else
		{
			text += "Theme: " + Theme + "\n" +
				"Subtheme: " + SubTheme;
		}			

		// Create text
		m_UI.push_back(new Text(m_WindowSurface, m_ScreenSize,
			glm::vec2(0.5f, 0.79f),
			WHITE, true,
			text));
	}
}

void Manager::CreateConfirmBrickSelection()
{
	const float rows = 10;
	const float columns = 5;

	// Calculate the bricks to display
	{
		m_BS_DisplayBricks.clear();

		m_ShowSelectedOnly = true;
		m_BS_DisplayBricks = GetDisplayableBricks(m_AllBricks, false, false, false);
		m_ShowSelectedOnly = false;
	}

	// Apply button
	{
		m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
			glm::vec2(0.8f, 0.93f), glm::vec2(0.99f, 0.98f),
			GREY, BLACK,
			"Save and Apply",
			[this]() {
				// Save the bricks and sets for future
				GottenBricksToCSV();
				GottenSetsToCSV();

				CheckPossibleSets();

				// Reset variables
				m_SS_SetNum.clear();

				// Load menu
				m_SS_ComingFromBS = false;
				m_SS_ShouldReloadSets = true;
				m_ScreenToLoad = Menus::SetSelection;
			}));
	}

	// Cancel button
	{
		m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
			glm::vec2(0.01f, 0.93f), glm::vec2(0.2f, 0.98f),
			GREY, BLACK,
			"Cancel",
			[this]() {
				m_ScreenToLoad = Menus::BrickSelection;
				m_BS_ShouldReloadBricks = true;
			}));
	}

	// Page selector
	{
		// 1 page
		{
			if (m_BS_Page > 1)
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.4f, 0.93f), glm::vec2(0.45f, 0.98f),
					GREY, BLACK,
					"<",
					[this]() {
						m_BS_Page--;
						m_ScreenToLoad = Menus::ConfirmBrickSelection;
					}));
			}

			if ((int)m_BS_DisplayBricks.size() > ((int)rows * (int)columns * m_BS_Page))
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.55f, 0.93f), glm::vec2(0.6f, 0.98f),
					GREY, BLACK,
					">",
					[this]() {
						m_BS_Page++;
						m_ScreenToLoad = Menus::ConfirmBrickSelection;
					}));
			}
		}

		// 10 page
		{
			if (m_BS_Page > 10)
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.345f, 0.93f), glm::vec2(0.395f, 0.98f),
					GREY, BLACK,
					"<<",
					[this]() {
						m_BS_Page -= 10;
						m_ScreenToLoad = Menus::ConfirmBrickSelection;
					}));
			}

			if ((int)m_BS_DisplayBricks.size() > ((int)rows * (int)columns * (m_BS_Page + 9)))
			{
				m_UI.push_back(new Button(m_WindowSurface, m_ScreenSize,
					glm::vec2(0.605f, 0.93f), glm::vec2(0.655f, 0.98f),
					GREY, BLACK,
					">>",
					[this]() {
						m_BS_Page += 10;
						m_ScreenToLoad = Menus::ConfirmBrickSelection;
					}));
			}
		}

		// Page counter
		m_UI.push_back(new Text(m_WindowSurface, m_ScreenSize,
			glm::vec2(0.5f, 0.95f),
			WHITE, true,
			std::to_string(m_BS_Page)));
	}

	// Tiles
	{
		if (m_BS_DisplayBricks.size() == 0)
		{
			m_UI.push_back(new Text(m_WindowSurface, m_ScreenSize,
				glm::vec2(0.5f, 0.5f),
				WHITE, true,
				"No bricks were selected"));
		}
		else
		{
			const float left_margin = 0.01f;
			const float right_margin = 0.995f;
			const float top_margin = 0.1f;
			const float bottom_margin = 0.92f;

			for (int y = 0; y < columns; y++)
			{
				for (int x = 0; x < rows; x++)
				{
					int index = x + (y * (int)rows) + ((m_BS_Page - 1) * (int)rows * (int)columns);
					if (index < (int)m_BS_DisplayBricks.size())
					{
						GottenBricks* item = &m_BS_DisplayBricks[index];

						// Create actual tile
						glm::vec2 TL = glm::vec2(
							left_margin + ((float)x / rows) * (right_margin - left_margin),
							top_margin + ((float)y / columns) * (bottom_margin - top_margin));

						glm::vec2 BR = glm::vec2(
							left_margin + (((float)x + 0.95f) / rows) * (right_margin - left_margin),
							top_margin + (((float)y + 0.95f) / columns) * (bottom_margin - top_margin));

						m_UI.push_back(new Tile(m_WindowSurface, m_ScreenSize,
							TL, BR,
							GREY, BLACK,
							item, true, &m_InventoryParts,
							[](GottenBricks* brick) {}));
					}
				}
			}
		}
	}

	// Text
	{
		m_UI.push_back(new Text(m_WindowSurface, m_ScreenSize,
			glm::vec2(0.5f, 0.05f),
			WHITE, true,
			"Check if all bricks are correct"));
	}
}

/// <summary>
/// File IO
/// </summary>
void Manager::OpenFiles()
{
	printf("Loading files, this could take a while.\n");

	const int FileAmount = 11;
	std::string str = "";
	std::ifstream stream;

	// inventory_parts
	if (LoadInventoryParts)
	{
		m_InventoryParts.reserve(1087706);

		printf("[1/%i] Loading inventory_parts\n", FileAmount);
		stream.open("Files/CSVs/inventory_parts.csv", std::ios::in | std::ios::binary);
		if (!stream)
		{
			printf("File inventory_parts.csv not found");
			abort();
		}
		stream.ignore(LLONG_MAX, '\n');
		while (true)
		{
			std::getline(stream, str);
			if (stream.eof())
				break;
			m_InventoryParts.push_back(InventoryParts(str));
		}
		stream.close();
	}

	// inventories
	if (LoadInventories)
	{
		m_Inventories.reserve(34765);

		printf("[2/%i] Loading inventories\n", FileAmount);
		stream.open("Files/CSVs/inventories.csv", std::ios::in | std::ios::binary);
		if (!stream)
		{
			printf("File inventories.csv not found");
			abort();
		}
		stream.ignore(LLONG_MAX, '\n');
		while (true)
		{
			std::getline(stream, str);
			if (stream.eof())
				break;
			m_Inventories.push_back(Inventories(str));
		}
		stream.close();
	}

	// sets
	if (LoadSets)
	{
		m_Sets.reserve(20480);

		printf("[3/%i] Loading sets\n", FileAmount);
		stream.open("Files/CSVs/sets.csv", std::ios::in | std::ios::binary);
		if (!stream)
		{
			printf("File sets.csv not found");
			abort();
		}
		stream.ignore(LLONG_MAX, '\n');
		while (true)
		{
			std::getline(stream, str);
			if (stream.eof())
				break;
			m_Sets.push_back(Sets(str));
		}
		stream.close();
	}
	
	// colors
	if (LoadColors)
	{
		m_Colors.reserve(217);

		printf("[4/%i] Loading colors\n", FileAmount);
		stream.open("Files/CSVs/colors.csv", std::ios::in | std::ios::binary);
		if (!stream)
		{
			printf("File colors.csv not found");
			abort();
		}
		stream.ignore(LLONG_MAX, '\n');
		while (true)
		{
			std::getline(stream, str);
			if (stream.eof())
				break;
			m_Colors.push_back(Colors(str));
		}
		stream.close();
	}

	// parts
	if (LoadParts)
	{
		m_Parts.reserve(49593);

		printf("[5/%i] Loading parts\n", FileAmount);
		stream.open("Files/CSVs/parts.csv", std::ios::in | std::ios::binary);
		if (!stream)
		{
			printf("File parts.csv not found");
			abort();
		}
		stream.ignore(LLONG_MAX, '\n');
		while (true)
		{
			std::getline(stream, str);
			if (stream.eof())
				break;
			m_Parts.push_back(Parts(str));
		}
		stream.close();
	}

	// themes
	if (LoadThemes)
	{
		m_Themes.reserve(459);

		printf("[6/%i] Loading themes\n", FileAmount);
		stream.open("Files/CSVs/themes.csv", std::ios::in | std::ios::binary);
		if (!stream)
		{
			printf("File themes.csv not found");
			abort();
		}
		stream.ignore(LLONG_MAX, '\n');
		while (true)
		{
			std::getline(stream, str);
			if (stream.eof())
				break;
			m_Themes.push_back(Themes(str));
		}
		stream.close();
	}
		
	// part_categories
	if (LoadPartCategories)
	{
		m_PartCategories.reserve(66);

		printf("[7/%i] Loading part_categories\n", FileAmount);
		stream.open("Files/CSVs/part_categories.csv", std::ios::in | std::ios::binary);
		if (!stream)
		{
			printf("File part_categories.csv not found");
			abort();
		}
		stream.ignore(LLONG_MAX, '\n');
		while (true)
		{
			std::getline(stream, str);
			if (stream.eof())
				break;
			m_PartCategories.push_back(PartCategories(str));
		}
		stream.close();
	}

	// part_relationships
	if (LoadPartRelationships)
	{
		m_PartRelationships.reserve(27756);

		printf("[8/%i] Loading part_relationships\n", FileAmount);
		stream.open("Files/CSVs/part_relationships.csv", std::ios::in | std::ios::binary);
		if (!stream)
		{
			printf("File part_relationships.csv not found");
			abort();
		}
		stream.ignore(LLONG_MAX, '\n');
		while (true)
		{
			std::getline(stream, str);
			if (stream.eof())
				break;
			m_PartRelationships.push_back(PartRelationships(str));
		}
		stream.close();
	}

	// elements
	if (LoadElements)
	{
		m_Elements.reserve(74937);

		printf("[9/%i] Loading elements\n", FileAmount);
		stream.open("Files/CSVs/elements.csv", std::ios::in | std::ios::binary);
		if (!stream)
		{
			printf("File elements.csv not found");
			abort();
		}
		stream.ignore(LLONG_MAX, '\n');
		while (true)
		{
			std::getline(stream, str);
			if (stream.eof())
				break;
			m_Elements.push_back(Elements(str));
		}
		stream.close();
	}

	// other_sets
	if (LoadMinifigSets)
	{
		printf("[10/%i] Loading other_sets\n", FileAmount);
		stream.open("Files/CSVs/other_sets.csv", std::ios::in | std::ios::binary);
		if (!stream)
		{
			printf("File other_sets.csv not found");
			abort();
		}
		stream.ignore(LLONG_MAX, '\n');
		while (true)
		{
			std::getline(stream, str);
			if (stream.eof())
				break;

			m_MinifigSets.push_back(std::string(str.substr(0, str.length() - 1)));
		}
		stream.close();
	}

	// inventory_sets
	if (LoadInventorySets)
	{
		m_InventorySets.reserve(3986);

		printf("[11/%i] Loading inventory_sets\n", FileAmount);
		stream.open("Files/CSVs/inventory_sets.csv", std::ios::in | std::ios::binary);
		if (!stream)
		{
			printf("File inventory_sets.csv not found");
			abort();
		}
		stream.ignore(LLONG_MAX, '\n');
		while (true)
		{
			std::getline(stream, str);
			if (stream.eof())
				break;
			m_InventorySets.push_back(InventorySets(str));
		}
		stream.close();
	}

	printf("All files have been loaded successfully\n\n");
}

void Manager::OpenCustomFiles()
{
	printf("Loading saved files from last time.\n");

	std::string str = "";
	std::ifstream stream;

	if (!LoadGottenBricks)
		return;

	// bricks
	{
		stream.open("Files/CSVs/gotten_bricks.csv", std::ios::in | std::ios::binary);
		if (!stream)
		{
			printf("File gotten_bricks.csv not found, starting without saved bricks\n\n");
			return;
		}
		else
		{
			printf("[1/2] Loading gotten bricks\n");
			std::vector<GottenBricks> temp;

			// Get values
			stream.ignore(LLONG_MAX, '\n');
			while (true)
			{
				std::getline(stream, str);
				if (stream.eof())
					break;

				temp.push_back(GottenBricks(str));
			}
			stream.close();

			// Set quantities within m_AllBricks
			for (GottenBricks brick : temp)
			{
				std::vector<GottenBricks>::iterator BrickIt = std::lower_bound(m_AllBricks.begin(), m_AllBricks.end(), brick);
				if (BrickIt != m_AllBricks.end())
				{
					int BrickIndex = BrickIt - m_AllBricks.begin();
					m_AllBricks[BrickIndex].Quantity = brick.Quantity;
				}
				else
				{
					printf("Brick not found, try switching to release!");
					abort();
				}
			}
		}
	}

	// sets
	{
		stream.open("Files/CSVs/gotten_sets.csv", std::ios::in | std::ios::binary);
		if (!stream)
		{
			printf("File gotten_sets.csv not found, starting without saved sets\n\n");
		}
		else
		{
			printf("[2/2] Loading gotten sets\n");

			char set_num[20 + 1] = { 0 };
			int quantity = 0;

			// Get values
			stream.ignore(LLONG_MAX, '\n');
			while (true)
			{
				std::getline(stream, str);
				if (stream.eof())
					break;

				// Get variables
				sscanf_s(str.c_str(), "%[^,],%i",
					set_num, sizeof(set_num),
					&quantity);

				std::vector<Sets>::iterator SetIt = std::lower_bound(m_Sets.begin(), m_Sets.end(), set_num);
				if (SetIt != m_Sets.end())
				{
					int SetIndex = SetIt - m_Sets.begin();
					m_Sets[SetIndex].Quantity = quantity;
				}
				else
				{
					printf("Set not found, try switching to release!");
					abort();
				}
			}
			stream.close();
		}
	}

	printf("All save files have been loaded successfully\n\n");
	m_CustomFilesLoaded = true;
}

void Manager::LoadImages()
{
	printf("Loading images, this will take a long time.\n");

	printf("Loading bricks.\n");
	{
		int i = 0;
		bool Print = false;
		for (InventoryParts Part : m_InventoryParts)
		{
			if (!Part.ImgURL.empty())
			{
				// Init variables
				std::string Type = Part.ImgURL.substr(Part.ImgURL.length() - 4);
				std::string Path = "Files/Images/Parts/" + Part.PartNum + "_" + std::to_string(Part.ColorID) + Type;

				// Check if file exists
				std::ifstream file(Path);
				if (!file.good())
				{
					// File doesn't exists, thus download the file
					URLDownloadToFile(NULL,
						std::wstring(Part.ImgURL.begin(), Part.ImgURL.end()).c_str(),
						std::wstring(Path.begin(), Path.end()).c_str(),
						0, NULL);

					if (Print)
					{
						printf("%2.2f%%\n", ((float)(i) / m_InventoryParts.size()) * 100.f);
						Print = false;
					}
				}
				file.close();
			}

			if (Print && i % 10000 == 0)
				printf("%2.2f%%\n", ((float)(i) / m_InventoryParts.size()) * 100.f);
			if (i % 100 == 0)
				Print = true;

			i++;
		}
	}

	printf("Loading sets.\n");
	{
		int i = 0;
		bool Print = false;
		for (Sets Set : m_Sets)
		{
			if (!Set.ImgURL.empty())
			{
				// Init variables
				std::string Type = Set.ImgURL.substr(Set.ImgURL.length() - 4);
				std::string Path = "Files/Images/Sets/" + Set.SetNum + Type;

				// Check if file exists
				std::ifstream file(Path);
				if (!file.good())
				{
					// File doesn't exists, thus download the file
					URLDownloadToFile(NULL,
						std::wstring(Set.ImgURL.begin(), Set.ImgURL.end()).c_str(),
						std::wstring(Path.begin(), Path.end()).c_str(),
						0, NULL);

					if (Print)
					{
						printf("%2.2f%%\n", ((float)(i) / m_Sets.size()) * 100.f);
						Print = false;
					}
				}
				file.close();
			}

			if (Print && i % 10000 == 0)
				printf("%2.2f%%\n", ((float)(i) / m_InventoryParts.size()) * 100.f);
			if (i % 10 == 0)
				Print = true;

			i++;
		}
	}

	printf("Done loading images\n\n");
}

void Manager::GottenBricksToCSV()
{
	// Make a new file and fill it with the contents of m_GottenBricks
	// Overwrite old file if needed
	printf("Saving gotten bricks.\n");

	// Get bricks to save
	std::vector<GottenBricks> BricksToSave = GetDisplayableBricks(m_AllBricks, false, false, true);

	if (BricksToSave.empty())
		printf("No gotten bricks selected.\n\n");
	else
	{
		FILE* stream;
		fopen_s(&stream, "Files/CSVs/gotten_bricks.csv", "w");
		if (!stream)
		{
			printf("File gotten_bricks.csv wasn't created!");
			abort();
		}

		// Add header to csv
		fprintf(stream, "part_num,color_id,quantity\n");

		// Add values to csv
		for (GottenBricks Brick : BricksToSave)
		{
			fprintf(stream, "%s,%i,%i\n",
				Brick.PartNum.c_str(),
				Brick.ColorID,
				Brick.Quantity);
		}
		fclose(stream);

		printf("Save successful.\n\n");
	}
}

void Manager::GottenSetsToCSV()
{
	// Make a new file and fill it with the contents of m_Sets
	// Overwrite old file if needed
	printf("Saving gotten sets.\n");

	// Get sets to save
	std::vector<Sets> SetsToSave = GetDisplayableSets(m_Sets, false, true, false);

	if (SetsToSave.empty())
		printf("No gotten sets selected.\n\n");
	else
	{
		FILE* stream;
		fopen_s(&stream, "Files/CSVs/gotten_sets.csv", "w");
		if (!stream)
		{
			printf("File gotten_sets.csv wasn't created!");
			abort();
		}

		// Add header to csv
		fprintf(stream, "set_num,quantity\n");

		// Add values to csv
		for (Sets Set : SetsToSave)
		{
			fprintf(stream, "%s,%i\n",
				Set.SetNum.c_str(),
				Set.Quantity);
		}
		fclose(stream);

		printf("Save successful.\n\n");
	}
}

/// <summary>
/// Miscellaneous
/// </summary>
void Manager::CreateListOfParts()
{
	printf("Creating a list of all parts.\n");
		
	// Add all elements to m_AllBricks
	for (Elements Element : m_Elements)
	{
		m_AllBricks.push_back(GottenBricks(Element, 0));
	}

	// Sort list
	std::sort(m_AllBricks.begin(), m_AllBricks.end());

	// Remove duplicates
	std::vector<GottenBricks> temp;
	GottenBricks PrevBrick = GottenBricks(Elements(), 0);
	for (GottenBricks Brick : m_AllBricks)
	{
		if (!(Brick == PrevBrick))
		{
			temp.push_back(Brick);
			PrevBrick = Brick;
		}
	}
	m_AllBricks = temp;

	// Skip this part as it takes ages for it to load in debug
	if (!DEBUG)
	{
		// Add all bricks instead of the items from only elements.csv
		int i = 0;
		for (InventoryParts Inv : m_InventoryParts)
		{
			// Check if it is already in all bricks
			GottenBricks Brick = GottenBricks(Elements(Inv.PartNum, Inv.ColorID), 0);
			if (!std::binary_search(m_AllBricks.begin(), m_AllBricks.end(), Brick))
			{
				// Calculate where to add it, so you don't have to sort the whole array every single time you add something
				std::vector<GottenBricks>::iterator it = std::lower_bound(m_AllBricks.begin(), m_AllBricks.end(), Brick);
				m_AllBricks.insert(it, GottenBricks(Brick, 0));
			}

			if (i % 100000 == 0)
				printf("%2.2f%%\n", ((float)(i) / m_InventoryParts.size()) * 100.f);

			i++;
		}
	}

	printf("Done making a list of all parts.\n\n");
}

void Manager::CheckPossibleSets()
{
	// Get bricks to check
	std::vector<GottenBricks> BricksGotten = GetDisplayableBricks(m_AllBricks, false, false, true);

	// Check which sets you can make
	int ID = -1;
	bool SetPossible = false;
	std::vector<int> PossibleSets;
	for (InventoryParts IPart : m_InventoryParts)
	{
		if (ID != IPart.InventoryID)
		{
			// Add set if gotten all bricks
			if (SetPossible)
				PossibleSets.push_back(ID);

			// Reset variables for next set
			ID = IPart.InventoryID;
			SetPossible = true;
		}
		
		if (SetPossible && !IPart.IsSpare)
		{
			// Check if you have the brick
			bool GotBrick = false;
			for (GottenBricks GPart : BricksGotten)
			{
				if (IPart.PartNum == GPart.PartNum && IPart.ColorID == GPart.ColorID && GPart.Quantity >= IPart.Quantity)
				{
					GotBrick = true;
					break;
				}
			}

			if (!GotBrick)
				SetPossible = false;
		}
	}

	// Turn vector of IDs into actual sets
	for (int InventoryID : PossibleSets)
	{
		//Get the inventory correlated to Inventory ID
		std::vector<Inventories>::iterator InvIt = std::lower_bound(m_Inventories.begin(), m_Inventories.end(), InventoryID);
		if (InvIt != m_Inventories.end())
		{
			int InvIndex = InvIt - m_Inventories.begin();

			std::vector<Sets>::iterator SetIt = std::find(m_Sets.begin(), m_Sets.end(), m_Inventories[InvIndex].SetNum);
			if (SetIt != m_Sets.end())
			{
				int SetIndex = SetIt - m_Sets.begin();

				m_PossibleSets.push_back(m_Sets[SetIndex]);
			}
		}
	}
}

std::vector<GottenBricks>  Manager::GetDisplayableBricks(
	std::vector<GottenBricks>& input,
	bool CheckColor,
	bool CheckPartNum,
	bool CheckQuantity)
{
	std::vector<GottenBricks> UseableBricks;
	for (GottenBricks& Brick : input)
	{
		bool DisplayBrick = true;

		// Selected
		if (DisplayBrick && (m_ShowSelectedOnly || CheckQuantity) && Brick.Quantity == 0)
			DisplayBrick = false;

		// Color
		if (CheckColor && DisplayBrick && m_BS_Color != -2 && Brick.ColorID != m_BS_Color)
			DisplayBrick = false;

		// PartNum
		if (CheckPartNum && DisplayBrick && !m_BS_PartNum.empty())
		{
			std::string str = Brick.PartNum;
			int length = (int)str.length();
			for (int i = 0; i < length; i++)
			{
				str[i] = std::tolower(str[i]);
			}

			if (str.find(m_BS_PartNum) == std::string::npos)			
				DisplayBrick = false;
		}

		if (DisplayBrick)
			UseableBricks.push_back(Brick);
	}

	return UseableBricks;
}

std::vector<Sets> Manager::GetDisplayableSets(
	std::vector<Sets> input,
	bool CheckSetNum,
	bool CheckQuantity,
	bool CheckMinifigSets)
{
	std::vector<Sets> UseableSets;
	int index = 0;
	for (Sets Set : input)
	{
		bool DisplaySet = true;

		// Selected
		if (DisplaySet && (m_ShowSelectedOnly || CheckQuantity) && Set.Quantity == 0)
			DisplaySet = false;

		// SetNum
		if (CheckSetNum && DisplaySet && !m_SS_SetNum.empty())
		{
			std::string str = Set.SetNum;
			int length = (int)str.length();
			for (int i = 0; i < length; i++)
			{
				str[i] = std::tolower(str[i]);
			}

			if (str.find(m_SS_SetNum) == std::string::npos)
				DisplaySet = false;			
		}

		// MinifigSets
		if (CheckMinifigSets && DisplaySet)
		{
			std::vector<std::string>::iterator StrIt = std::find(m_MinifigSets.begin(), m_MinifigSets.end(), Set.SetNum);
			if (StrIt != m_MinifigSets.end())
				DisplaySet = false;
		}

		if (DisplaySet)
			UseableSets.push_back(Set);

		index++;
	}

	return UseableSets;
}

/// <summary>
/// main
/// </summary>
int main(int, char**)
{	
	Manager manager;

	// Open files before init renderer, as loading the files could take a while.
	// We don't want the user to see a blank screen without being able to click on anything.
	manager.OpenFiles();
	manager.CreateListOfParts();
	manager.OpenCustomFiles();

	int ExitValue = manager.InitRenderer();
	if (ExitValue != 0)
	{
		printf("Error in Init: return value %i", ExitValue);
		abort();
	}

	if (DEBUG)
	{
		// Skip the start screen if don't have any bricks
		if (manager.m_CustomFilesLoaded)
			manager.m_ScreenToLoad = Menus::StartScreen;
		else
			manager.m_ScreenToLoad = Menus::BrickSelection;
	}
	else
		manager.m_ScreenToLoad = Menus::LoadImagesScreen;

	while (!manager.m_ExitApp)
	{
		// Begin delta time
		auto start = std::chrono::steady_clock::now();

		manager.ClearScreen();

		manager.UpdateUI();

		if (manager.m_ShouldRender)
		{
			manager.RenderUI();
			manager.RenderScreen();
		}

		manager.CheckEvents();

		// End delta time
		auto end = std::chrono::steady_clock::now();
		std::chrono::duration<float> elapsed_chrono_time = end - start;
		manager.m_DeltaTime = elapsed_chrono_time.count();

		manager.CapFPS(60);
	}

	// Quit game
	manager.RemoveUI();
	SDL_DestroyRenderer(manager.m_Renderer);
	SDL_FreeSurface(manager.m_WindowSurface);
	SDL_DestroyWindow(manager.m_Window);
	SDL_Quit();

	return 0;
}