#include "Button.h"
#include "../Main.h"

Button::Button(SDL_Surface* surface,
	glm::ivec2 screen_size,
	glm::vec2 top_left,
	glm::vec2 bottom_right,
	glm::ivec4 color,
	glm::ivec4 text_color,
	std::string text,
	std::function<void()> func) :

	UIParent(surface, screen_size),
	Color(color),
	TextColor(text_color),
	Text(text),
	OnClick(func)
{
	TL = glm::ivec2(top_left.x * (float)ScreenSize.x, top_left.y * (float)ScreenSize.y);
	BR = glm::ivec2(bottom_right.x * (float)ScreenSize.x, bottom_right.y * (float)ScreenSize.y);
}

bool Button::Update(glm::ivec2 MousePos, bool MouseButtonDown)
{
	bool BeginHover = Hovered;
	bool BeginClick = Clicked;

	// Check if button is hovered
	bool hovered = MousePos.x > TL.x && MousePos.x < BR.x && MousePos.y > TL.y && MousePos.y < BR.y;
	if (hovered && MouseButtonDown)
	{
		if (Hovered)
			Clicked = true;
	}
	else if (hovered)
	{
		Hovered = true;
		if (Clicked)
		{
			Clicked = false;
			OnClick();
		}
	}
	else
	{
		Hovered = false;
		Clicked = false;
	}

	return BeginHover != Hovered || BeginClick != Clicked;
}

void Button::Render()
{
	glm::ivec4 color = Color;
	if (Clicked)
		color -= glm::ivec4(50, 50, 50, 0);
	else if (Hovered)
		color += glm::ivec4(50, 50, 50, 0);

	// Render Rect
	SDL_Rect Rect = SDL_Rect();
	Rect.x = TL.x;
	Rect.y = TL.y;
	Rect.w = BR.x - TL.x;
	Rect.h = BR.y - TL.y;
	SDL_FillRect(ManagerSurface, &Rect, SDL_MapRGB(ManagerSurface->format, color.r, color.g, color.b));

	// Render Text
	glm::ivec2 pos = glm::ivec2(
		TL.x + (Rect.w / 2),
		TL.y + (Rect.h / 2)
	);
	DrawString(Text, pos, TextColor, true, (float)TEXTSIZE);
}