#include "DropDown.h"
#include "../Main.h"

DropDown::DropDown(SDL_Surface* surface,
	glm::ivec2 screen_size,
	glm::vec2 top_left,
	glm::vec2 bottom_right,
	glm::ivec4 color,
	glm::ivec4 text_color,
	std::string default_text,
	std::vector<DropDownItem> items,
	int selected,
	std::function<void(int item)> func) :

	UIParent(surface, screen_size),
	Color(color),
	TextColor(text_color),
	DefaultText(default_text),
	CurrentText(default_text),
	OnClick(func)
{
	TL = glm::ivec2(top_left.x * (float)ScreenSize.x, top_left.y * (float)ScreenSize.y);
	BR = glm::ivec2(bottom_right.x * (float)ScreenSize.x, bottom_right.y * (float)ScreenSize.y);

	// Add all options
	for (DropDownItem item : items)
	{
		Items.push_back(DropDownItem(item.Name, item.Amount));
	}

	// Set current text
	if (selected != -2)
		CurrentText = Items[selected + 1].Name;
}

bool DropDown::Update(glm::ivec2 MousePos, bool MouseButtonDown)
{
	bool ScreenUpdate = false;

	if (Scrolled != PrevScrolled)
	{
		PrevScrolled = Scrolled;
		ScreenUpdate = true;
	}

	// DropDown itself
	ScreenUpdate |= UpdateItem(MousePos, MouseButtonDown, Hovered, Clicked, TL, BR, true, -1, " ");

	// Close dropdown if clicked outside of main button
	bool hovered = MousePos.x > TL.x && MousePos.x < BR.x&& MousePos.y > TL.y && MousePos.y < BR.y;
	for (DropDownItem& Item : Items)
	{
		hovered |= Item.Hovered;
	}
	if (!hovered && MouseButtonDown)
	{
		Openened = false;
		ScreenUpdate = true;
	}

	// DropDown items
	if (Openened)
	{
		// Remove not needed items
		std::vector<DropDownItem> DisplayItems;
		for (DropDownItem& Item : Items)
		{
			if (Item.Amount != 0)
				DisplayItems.push_back(Item);
		}

		// Update items
		int count = 0;
		glm::ivec2 tl = TL + glm::ivec2(0, 65);
		glm::ivec2 br = BR + glm::ivec2(0, 50);
		for (DropDownItem& Item : DisplayItems)
		{
			if (count >= MAXITEMS + Scrolled)
				break;

			if (count >= Scrolled)
			{
				// Get index from Items list
				std::vector<DropDownItem>::iterator DropDownIt = std::find(Items.begin(), Items.end(), Item);
				if (DropDownIt != Items.end())
				{
					int index = DropDownIt - Items.begin();
					DropDownItem &DDI = Items[index];

					ScreenUpdate |= UpdateItem(MousePos, MouseButtonDown, DDI.Hovered, DDI.Clicked, tl, br, false, index - 1, DDI.Name);

					tl += OFFSET;
					br += OFFSET;
				}
			}

			count++;
		}
	}

	return ScreenUpdate;
}

void DropDown::Render()
{
	// DropDown itself
	std::string text = CurrentText + (Openened ? " -" : " +");
	RenderItem(Color, text, Hovered, Clicked, TL, BR, true);

	// DropDown items
	if (Openened)
	{
		// Remove not needed items
		std::vector<DropDownItem> DisplayItems;
		for (DropDownItem& Item : Items)
		{
			if (Item.Amount != 0)
				DisplayItems.push_back(Item);
		}
		
		// Render items
		int count = 0;
		glm::ivec2 tl = TL + glm::ivec2(0, 65);
		glm::ivec2 br = BR + glm::ivec2(0, 50);
		for (DropDownItem &Item : DisplayItems)
		{
			if (count >= MAXITEMS + Scrolled)
				break;

			if (count >= Scrolled)
			{
				std::string name = Item.Name;
				if (Item.Amount != -1)
					name += " (" + std::to_string(Item.Amount) + ")";

				RenderItem(Color, name, Item.Hovered, Item.Clicked, tl, br, false);

				tl += OFFSET;
				br += OFFSET;
			}

			count++;
		}
	}
}

bool DropDown::UpdateItem(glm::ivec2 &MousePos,
	bool &MouseButtonDown,
	bool& Hovered,
	bool& Clicked,
	glm::ivec2& TL,
	glm::ivec2& BR,
	bool MainButton,
	int item,
	std::string Text)
{
	bool BeginHover = Hovered;
	bool BeginClick = Clicked;

	// Check if button is hovered
	bool hovered = MousePos.x > TL.x && MousePos.x < BR.x&& MousePos.y > TL.y && MousePos.y < BR.y;
	if (hovered && MouseButtonDown)
	{
		if (Hovered)
			Clicked = true;
	}
	else if (hovered)
	{
		Hovered = true;
		if (Clicked)
		{
			Clicked = false;

			if (MainButton)
				Openened = !Openened;
			else
			{
				Openened = false;
				OnClick(item);

				if (item == -1)
					CurrentText = DefaultText;
				else
					CurrentText = Text;
			}
		}
	}
	else
	{
		Hovered = false;
		Clicked = false;
	}

	return BeginHover != Hovered || BeginClick != Clicked;
}

void DropDown::RenderItem(glm::ivec4 &Color,
	std::string &Text,
	bool& Hovered,
	bool& Clicked,
	glm::ivec2& TL,
	glm::ivec2& BR,
	bool MainButton)
{
	glm::ivec4 color = Color;
	if (!MainButton)
		color += glm::ivec4(10, 10, 10, 0);

	if (Clicked)
		color -= glm::ivec4(50, 50, 50, 0);
	else if (Hovered)
		color += glm::ivec4(50, 50, 50, 0);

	// Render Rect
	SDL_Rect Rect = SDL_Rect();
	Rect.x = TL.x;
	Rect.y = TL.y;
	Rect.w = BR.x - TL.x;
	Rect.h = BR.y - TL.y;
	SDL_FillRect(ManagerSurface, &Rect, SDL_MapRGB(ManagerSurface->format, color.r, color.g, color.b));

	// Render Text
	glm::ivec2 pos = glm::ivec2(
		TL.x + (Rect.w / 2),
		TL.y + (Rect.h / 2)
	);
	float Textsize = (float)TEXTSIZE - 1.f;
	if (!MainButton)
		Textsize -= 0.f;
	DrawString(Text, pos, TextColor, true, Textsize);
}

void DropDown::Scroll(int movement)
{
	// Count valid items
	int size = 0;
	for (DropDownItem& Item : Items)
	{
		if (Item.Amount != 0)
			size++;
	}

	int max = size - MAXITEMS;
	if (max > 0)
	{
		Scrolled -= movement;
		Scrolled = CLAMP(Scrolled, 0, max);
	}
}