#pragma once
#include "UIParent.h"

#include <functional>
#include <vector>

constexpr glm::ivec2 OFFSET = glm::ivec2(0, 50);
constexpr int MAXITEMS = 15;

class DropDownItem
{
public:
	DropDownItem(std::string name, int amount) : Name(name), Amount(amount) {}

	std::string Name = "";
	int Amount = -1;

	bool Hovered = false;
	bool Clicked = false;

	bool operator==(const DropDownItem& a)
	{
		return Name == a.Name;
	}
};

class DropDown : public UIParent
{
public:
	// Methods
	DropDown(SDL_Surface* surface,
		glm::ivec2 screen_size,
		glm::vec2 top_left,
		glm::vec2 bottom_right,
		glm::ivec4 color,
		glm::ivec4 text_color,
		std::string default_text,
		std::vector<DropDownItem> items,
		int selected,
		std::function<void(int item)> func);

	bool Update(glm::ivec2 MousePos, bool MouseButtonDown) override;
	void Render() override;

	bool UpdateItem(glm::ivec2& MousePos,
		bool& MouseButtonDown,
		bool &Hovered,
		bool &Clicked,
		glm::ivec2 &TL,
		glm::ivec2 &BR,
		bool MainButton,
		int item,
		std::string Text);
	void RenderItem(glm::ivec4 &Color,
		std::string &Text,
		bool& Hovered,
		bool& Clicked,
		glm::ivec2& TL,
		glm::ivec2& BR,
		bool MainButton);
	void Scroll(int movement);

	// Variables
	glm::ivec4 Color;
	const glm::ivec4 TextColor;
	std::string DefaultText;
	std::string CurrentText = "";
	std::vector<DropDownItem> Items;
	const std::function<void(int item)> OnClick;
	bool Openened = false;

private:
	bool Hovered = false;
	bool Clicked = false;
	glm::ivec2 TL = glm::ivec2();
	glm::ivec2 BR = glm::ivec2();
	int Scrolled = 0;
	int PrevScrolled = 0;
};