#include "EditableTextbox.h"

EditableTextbox::EditableTextbox(SDL_Surface* surface,
	glm::ivec2 screen_size,
	glm::vec2 top_left,
	glm::vec2 bottom_right,
	glm::ivec4 color,
	glm::ivec4 default_text_color,
	glm::ivec4 text_color,
	std::string default_text,
	std::string starting_text,
	bool should_edit,
	int max_length) :

	UIParent(surface, screen_size),
	Color(color),
	DefaultTextColor(default_text_color),
	TextColor(text_color),
	DefaultText(default_text),
	CurrentText(starting_text),
	PrevText(starting_text),
	Editing(should_edit),
	MaxLength(max_length)
{
	TL = glm::ivec2(top_left.x * (float)ScreenSize.x, top_left.y * (float)ScreenSize.y);
	BR = glm::ivec2(bottom_right.x * (float)ScreenSize.x, bottom_right.y * (float)ScreenSize.y);
}

bool EditableTextbox::Update(glm::ivec2 MousePos, bool MouseButtonDown)
{
	bool BeginHover = Hovered;
	bool BeginClick = Clicked;
	bool ShouldRender = false;

	// Check if button is hovered
	bool hovered = MousePos.x > TL.x && MousePos.x < BR.x&& MousePos.y > TL.y && MousePos.y < BR.y;
	if (hovered && MouseButtonDown)
	{
		if (Hovered)
			Clicked = true;
	}
	else if (hovered)
	{
		Hovered = true;
		if (Clicked)
		{
			Clicked = false;
			Editing = !Editing;
			if (!Editing)
				DisplayCursor = false;
		}
	}
	else
	{
		Hovered = false;
		Clicked = false;
	}

	// Close textbox if clicked outside of box
	if (!hovered && MouseButtonDown)
	{
		Editing = false;
		DisplayCursor = false;
		ShouldRender = true;
	}

	if (CurrentText != PrevText)
	{
		PrevText = CurrentText;
		ShouldRender = true;
	}

	if (DisplayCursor != PrevDisplayCursor)
	{
		PrevDisplayCursor = DisplayCursor;
		ShouldRender = true;
	}

	return ShouldRender || BeginHover != Hovered || BeginClick != Clicked;
}

void EditableTextbox::Render()
{
	glm::ivec4 color = Color;
	if (Clicked)
		color -= glm::ivec4(50, 50, 50, 0);
	else if (Hovered || Editing)
		color += glm::ivec4(50, 50, 50, 0);

	// Render Rect
	SDL_Rect Rect = SDL_Rect();
	Rect.x = TL.x;
	Rect.y = TL.y;
	Rect.w = BR.x - TL.x;
	Rect.h = BR.y - TL.y;
	SDL_FillRect(ManagerSurface, &Rect, SDL_MapRGB(ManagerSurface->format, color.r, color.g, color.b));

	// Render Text
	glm::ivec2 pos = glm::ivec2(
		TL.x + (Rect.w / 2),
		TL.y + (Rect.h / 2)
	);

	glm::ivec4 text_color = DefaultTextColor;
	std::string text = DefaultText;
	if (!CurrentText.empty() || DisplayCursor)
	{
		text_color = TextColor;
		text = CurrentText;
	}

	if (DisplayCursor)
		text += 'I';

	DrawString(text, pos, text_color, true, 3.f);
}

std::string EditableTextbox::AddCharacter(SDL_Keycode character)
{
	if (character == SDLK_RETURN || character == SDLK_KP_ENTER)
	{
		Editing = false;
		DisplayCursor = false;
		PrevText.clear();	// Force an screen render
		return CurrentText;
	}

	if (character == SDLK_BACKSPACE)
	{
		if (!CurrentText.empty())
			CurrentText.pop_back();
		return CurrentText;
	}

	if ((int)CurrentText.size() < MaxLength)
	{
		// Check if valid characters
		if ((character >= 'a' && character <= 'z') ||
			(character >= '0' && character <= '9') ||
			character == ' ')
		{
			CurrentText.push_back(character);
			return CurrentText;
		}

		// Check keypad numbers and other symbols
		char Char = ' ';
		switch (character)
		{
		case SDLK_KP_0: Char = '0'; break;
		case SDLK_KP_1: Char = '1'; break;
		case SDLK_KP_2: Char = '2'; break;
		case SDLK_KP_3: Char = '3'; break;
		case SDLK_KP_4: Char = '4'; break;
		case SDLK_KP_5: Char = '5'; break;
		case SDLK_KP_6: Char = '6'; break;
		case SDLK_KP_7: Char = '7'; break;
		case SDLK_KP_8: Char = '8'; break;
		case SDLK_KP_9: Char = '9'; break;
		case SDLK_MINUS: case SDLK_KP_MINUS: Char = '-'; break;
		default: break;
		}
		if (Char != ' ')
		{
			CurrentText.push_back(Char);
			return CurrentText;
		}
	}

	return CurrentText;
}