#include "Tile.h"
#include "../Main.h"

// GottenBricks tile
Tile::Tile(SDL_Surface* surface,
	glm::ivec2 screen_size,
	glm::vec2 top_left,
	glm::vec2 bottom_right,
	glm::ivec4 color,
	glm::ivec4 text_color,
	GottenBricks* item,
	bool show_quantity,
	std::vector<InventoryParts>* inventory_parts,
	std::function<void(GottenBricks* item)> func) :

	UIParent(surface, screen_size),
	Color(color),
	TextColor(text_color),
	UsingBricks(true),
	GB_Item(item),
	ShowQuantity(show_quantity),
	GB_OnClick(func)
{
	Init(top_left, bottom_right, inventory_parts);
}

// Sets tile
Tile::Tile(SDL_Surface* surface,
	glm::ivec2 screen_size,
	glm::vec2 top_left,
	glm::vec2 bottom_right,
	glm::ivec4 color,
	glm::ivec4 text_color,
	Sets* item,
	bool show_quantity,
	std::function<void(Sets* item)> func) :

	UIParent(surface, screen_size),
	Color(color),
	TextColor(text_color),
	UsingBricks(false),
	S_Item(item),
	ShowQuantity(show_quantity),
	S_OnClick(func)
{
	Init(top_left, bottom_right, nullptr);
}

Tile::~Tile()
{
	SDL_FreeSurface(Surface);
	FreeImage(Data);
}

void Tile::Init(glm::vec2 top_left, glm::vec2 bottom_right, std::vector<InventoryParts>* inventory_parts)
{
	CalcTL_BR(TL, BR, top_left, bottom_right, ScreenSize);

	// Get URL
	std::string URL = "";
	if (UsingBricks)
	{
		std::vector<InventoryParts>::iterator IPIt = std::find(inventory_parts->begin(), inventory_parts->end(), *GB_Item);
		if (IPIt != inventory_parts->end())
		{
			int index = IPIt - inventory_parts->begin();
			URL = inventory_parts->at(index).ImgURL;
		}
	}
	else
		URL = S_Item->ImgURL;

	Surface = LoadImageFromURL(&Data, URL, "", GB_Item, S_Item);
}

bool Tile::Update(glm::ivec2 MousePos, bool MouseButtonDown)
{
	bool BeginHover = Hovered;
	bool BeginClick = Clicked;

	// Check if button is hovered
	bool hovered = MousePos.x > TL.x && MousePos.x < BR.x&& MousePos.y > TL.y && MousePos.y < BR.y;
	if (hovered && MouseButtonDown)
	{
		if (Hovered)
			Clicked = true;
	}
	else if (hovered)
	{
		Hovered = true;
		if (Clicked)
		{
			Clicked = false;
			if (UsingBricks)
				GB_OnClick(GB_Item);
			else
				S_OnClick(S_Item);
		}
	}
	else
	{
		Hovered = false;
		Clicked = false;
	}

	return BeginHover != Hovered || BeginClick != Clicked;
}

void Tile::Render()
{
	glm::ivec4 color = Color;
	if (Clicked)
		color -= glm::ivec4(50, 50, 50, 0);
	else if (Hovered)
		color += glm::ivec4(50, 50, 50, 0);

	// Render Rect
	SDL_Rect Rect = SDL_Rect();
	Rect.x = TL.x;
	Rect.y = TL.y;
	Rect.w = BR.x - TL.x;
	Rect.h = BR.y - TL.y;
	SDL_FillRect(ManagerSurface, &Rect, SDL_MapRGB(ManagerSurface->format, color.r, color.g, color.b));

	// Render title
	{
		glm::ivec2 pos = glm::ivec2(
			TL.x + (Rect.w / 2),
			TL.y + (Rect.h - 10)
		);
		DrawString(UsingBricks ? GB_Item->PartNum : S_Item->SetNum, pos, TextColor, true, 2.f);
	}

	// Render quantity
	if (ShowQuantity)
	{
		int quantity = UsingBricks ? GB_Item->Quantity : S_Item->Quantity;
		if (quantity > 0)
		{
			glm::ivec2 pos = glm::ivec2(
				TL.x + (Rect.w / 2),
				TL.y + (Rect.h - 32)
			);

			char str[10];
			sprintf_s(str, "Q:%i", quantity);
			DrawString(str, pos, TextColor, true, 3.f);
		}
	}

	// Render image
	if (Surface != nullptr)
	{
		SDL_Rect SourceRect = SDL_Rect();
		SourceRect.x = 0;
		SourceRect.y = 0;
		SourceRect.w = Surface->w;
		SourceRect.h = Surface->h;

		SDL_Rect DestRect = SDL_Rect();
		DestRect.x = TL.x + 20;
		DestRect.y = TL.y + 5;
		DestRect.w = Rect.w - 40;
		DestRect.h = Rect.h - 50;

		SDL_BlitScaled(Surface, &SourceRect, ManagerSurface, &DestRect);
	}
}