#include "UIParent.h"
#include "../Main.h"

#define STB_IMAGE_IMPLEMENTATION
#include <algorithm>
#include <fstream>
#include "../lib/stb_image.h"

UIParent::UIParent(SDL_Surface* surface,
	glm::ivec2 screen_size) :

	ManagerSurface(surface),
	ScreenSize(screen_size)
{}

void UIParent::DrawString(std::string str, glm::ivec2 pos, glm::ivec4 color, bool centered, float size)
{
	int StrLength = str.length();
	int LineLength = 0;
	pos /= size;
	int CharPos = 0;
	glm::ivec2 DrawPos = pos;
	int Lines = 1;

	for (int c = 0; c < StrLength; c++)
	{
		if (str[c] == '\n')
			break;

		LineLength++;
	}

	if (centered)
	{
		for (int c = 0; c < StrLength; c++)
		{
			if (str[c] == '\n')
				Lines++;
		}
	}

	for (int i = 0; i < StrLength; i++)
	{
		if (str[i] == '\n')
		{
			DrawPos.y += 8;
			CharPos = 0;
			LineLength = 0;

			for (int c = i + 1; c < StrLength; c++)
			{
				if (str[c] == '\n')
					break;

				LineLength++;
			}

		}
		else
		{
			if (!centered)
				DrawChar(str[i], glm::ivec2(DrawPos.x + (CharPos * 6), DrawPos.y), color, size);
			else
			{
				DrawChar(str[i], glm::ivec2(
					DrawPos.x + (int)(((float)CharPos - ((float)LineLength / 2.f)) * 6.f),
					DrawPos.y - (int)(((float)Lines / 2.f) * 7.f)),
					color, size);
			}
			CharPos++;
		}
	}
}

void UIParent::DrawChar(const char character, glm::ivec2 pos, glm::ivec4 color, float size)
{
	std::string pixel;

	// Get which pixels should be written
	switch (character)
	{
		// Uppercase letters
	case 'A': pixel = "::o::  :o:o:  o:::o  ooooo  o:::o  o:::o  o:::o"; break;
	case 'B': pixel = "oooo:  o:::o  o:::o  oooo:  o:::o  o:::o  oooo:"; break;
	case 'C': pixel = ":ooo:  o:::o  o::::  o::::  o::::  o:::o  :ooo:"; break;
	case 'D': pixel = "ooo::  o::o:  o:::o  o:::o  o:::o  o::o:  ooo::"; break;
	case 'E': pixel = "ooooo  o::::  o::::  ooo::  o::::  o::::  ooooo"; break;
	case 'F': pixel = "ooooo  o::::  o::::  ooo::  o::::  o::::  o::::"; break;
	case 'G': pixel = ":ooo:  o:::o  o::::  o:ooo  o:::o  o:::o  :oooo"; break;
	case 'H': pixel = "o:::o  o:::o  o:::o  ooooo  o:::o  o:::o  o:::o"; break;
	case 'I': pixel = ":ooo:  ::o::  ::o::  ::o::  ::o::  ::o::  :ooo:"; break;
	case 'J': pixel = "::::o  ::::o  ::::o  ::::o  o:::o  o:::o  :ooo:"; break;
	case 'K': pixel = "o:::o  o::o:  o:o::  ooo::  o::o:  o:::o  o:::o"; break;
	case 'L': pixel = "o::::  o::::  o::::  o::::  o::::  o::::  ooooo"; break;
	case 'M': pixel = "o:::o  oo:oo  o:o:o  o:o:o  o:o:o  o:::o  o:::o"; break;
	case 'N': pixel = "o:::o  oo::o  o:o:o  o::oo  o:::o  o:::o  o:::o"; break;
	case 'O': pixel = ":ooo:  o:::o  o:::o  o:::o  o:::o  o:::o  :ooo:"; break;
	case 'P': pixel = "oooo:  o:::o  o:::o  oooo:  o::::  o::::  o::::"; break;
	case 'Q': pixel = ":ooo:  o:::o  o:::o  o:::o  o:o:o  o::oo  :oooo"; break;
	case 'R': pixel = "oooo:  o:::o  o:::o  oooo:  o:::o  o:::o  o:::o"; break;
	case 'S': pixel = ":ooo:  o:::o  o::::  :ooo:  ::::o  o:::o  :ooo:"; break;
	case 'T': pixel = "ooooo  ::o::  ::o::  ::o::  ::o::  ::o::  ::o::"; break;
	case 'U': pixel = "o:::o  o:::o  o:::o  o:::o  o:::o  o:::o  :ooo:"; break;
	case 'V': pixel = "o:::o  o:::o  o:::o  o:::o  o:::o  :o:o:  ::o::"; break;
	case 'W': pixel = "o:::o  o:::o  o:::o  o:o:o  o:o:o  o:o:o  oo:oo"; break;
	case 'X': pixel = "o:::o  o:::o  :o:o:  ::o::  :o:o:  o:::o  o:::o"; break;
	case 'Y': pixel = "o:::o  o:::o  :o:o:  ::o::  ::o::  ::o::  ::o::"; break;
	case 'Z': pixel = "ooooo  ::::o  :::o:  ::o::  :o:::  o::::  ooooo"; break;

		// Lowercase letters
	case 'a': pixel = ":::::  :::::  :ooo:  ::::o  :oooo  o:::o  :oooo"; break;
	case 'b': pixel = "o::::  o::::  o:oo:  oo::o  o:::o  o:::o  oooo:"; break;
	case 'c': pixel = ":::::  :::::  :oooo  o::::  o::::  o::::  :oooo"; break;
	case 'd': pixel = "::::o  ::::o  :oo:o  o::oo  o:::o  o:::o  :oooo"; break;
	case 'e': pixel = ":::::  :::::  :ooo:  o:::o  ooooo  o::::  :ooo:"; break;
	case 'f': pixel = "::oo:  :o::o  :o:::  ooo::  :o:::  :o:::  :o:::"; break;
	case 'g': pixel = ":::::  :::::  :oooo  o:::o  :oooo  ::::o  :ooo:"; break;
	case 'h': pixel = "o::::  o::::  o:oo:  oo::o  o:::o  o:::o  o:::o"; break;
	case 'i': pixel = "::o::  :::::  ::o::  :oo::  ::o::  ::o::  :ooo:"; break;
	case 'j': pixel = ":::o:  :::::  ::oo:  :::o:  :::o:  o::o:  :oo::"; break;
	case 'k': pixel = "o::::  o::::  o::o:  o:o::  oo:::  o:o::  o::o:"; break;
	case 'l': pixel = ":oo::  ::o::  ::o::  ::o::  ::o::  ::o::  :ooo:"; break;
	case 'm': pixel = ":::::  :::::  oo:o:  o:o:o  o:o:o  o:o:o  o:o:o"; break;
	case 'n': pixel = ":::::  :::::  o:oo:  oo::o  o:::o  o:::o  o:::o"; break;
	case 'o': pixel = ":::::  :::::  :ooo:  o:::o  o:::o  o:::o  :ooo:"; break;
	case 'p': pixel = ":::::  :::::  oooo:  o:::o  oooo:  o::::  o::::"; break;
	case 'q': pixel = ":::::  :::::  :oo:o  o::oo  :oooo  ::::o  ::::o"; break;
	case 'r': pixel = ":::::  :::::  o:oo:  oo::o  o::::  o::::  o::::"; break;
	case 's': pixel = ":::::  :::::  :oooo  o::::  :ooo:  ::::o  oooo:"; break;
	case 't': pixel = ":o:::  :o:::  ooo::  :o:::  :o:::  :o::o  ::oo:"; break;
	case 'u': pixel = ":::::  :::::  o:::o  o:::o  o:::o  o::oo  :oo:o"; break;
	case 'v': pixel = ":::::  :::::  o:::o  o:::o  o:::o  :o:o:  ::o::"; break;
	case 'w': pixel = ":::::  :::::  o:::o  o:::o  o:o:o  o:o:o  :o:o:"; break;
	case 'x': pixel = ":::::  :::::  o:::o  :o:o:  ::o::  :o:o:  o:::o"; break;
	case 'y': pixel = ":::::  :::::  o:::o  o:::o  :oooo  ::::o  :ooo:"; break;
	case 'z': pixel = ":::::  :::::  ooooo  :::o:  ::o::  :o:::  ooooo"; break;

		// Numbers
	case '0': pixel = ":ooo:  o:::o  o::oo  o:o:o  oo::o  o:::o  :ooo:"; break;
	case '1': pixel = "::o::  :oo::  ::o::  ::o::  ::o::  ::o::  :ooo:"; break;
	case '2': pixel = ":ooo:  o:::o  ::::o  :::o:  ::o::  :o:::  ooooo"; break;
	case '3': pixel = "ooooo  :::o:  ::o::  :::o:  ::::o  o:::o  :ooo:"; break;
	case '4': pixel = ":::o:  ::oo:  :o:o:  o::o:  ooooo  :::o:  :::o:"; break;
	case '5': pixel = "ooooo  o::::  oooo:  ::::o  ::::o  o:::o  :ooo:"; break;
	case '6': pixel = "::oo:  :o:::  o::::  oooo:  o:::o  o:::o  :ooo:"; break;
	case '7': pixel = "ooooo  ::::o  :::o:  ::o::  :o:::  :o:::  :o:::"; break;
	case '8': pixel = ":ooo:  o:::o  o:::o  :ooo:  o:::o  o:::o  :ooo:"; break;
	case '9': pixel = ":ooo:  o:::o  o:::o  :oooo  ::::o  :::o:  :oo::"; break;

		// Symbols
	case ' ': return;
	case '-': pixel = ":::::  :::::  :::::  ooooo  :::::  :::::  :::::"; break;
	case '+': pixel = ":::::  ::o::  ::o::  ooooo  ::o::  ::o::  :::::"; break;
	case ':': pixel = ":::::  ::o::  :::::  :::::  :::::  ::o::  :::::"; break;
	case '<': pixel = ":::::  ::o::  :o:::  o::::  :o:::  ::o::  :::::"; break;
	case '>': pixel = ":::::  ::o::  :::o:  ::::o  :::o:  ::o::  :::::"; break;
	case '(': pixel = "::::o  :::o:  ::o::  ::o::  ::o::  :::o:  ::::o"; break;
	case ')': pixel = "o::::  :o:::  ::o::  ::o::  ::o::  :o:::  o::::"; break;
	case '/': pixel = ":::o:  :::o:  ::o::  ::o::  ::o::  :o:::  :o:::"; break;
	case '[': pixel = "::oo:  ::o::  ::o::  ::o::  ::o::  ::o::  ::oo:"; break;
	case ']': pixel = ":oo::  ::o::  ::o::  ::o::  ::o::  ::o::  :oo::"; break;
	case '.': pixel = ":::::  :::::  :::::  :::::  :::::  :::::  ::o::"; break;
	case ',': pixel = ":::::  :::::  :::::  :::::  :::::  ::o::  ::o::"; break;
	case '?': pixel = ":ooo:  o:::o  ::::o  :::o:  ::o::  :::::  ::o::"; break;
	default: printf("Character '%c' was not able to be drawn!\n", character); return;
	}

	// Draw to screen
	for (int i = 0; i < 47; i++)
	{
		if (pixel[i] != ':' && pixel[i] != ' ')
		{
			SDL_Rect Rect = SDL_Rect();
			Rect.x = (int)size * (pos.x + i % 7);
			Rect.y = (int)size * (pos.y + (int)floor((float)i / 7.f));
			Rect.w = (int)size;
			Rect.h = (int)size;
			SDL_FillRect(ManagerSurface, &Rect, SDL_MapRGB(ManagerSurface->format, color.r, color.g, color.b));
		}
	}
}

void UIParent::CalcTL_BR(glm::ivec2& TL, glm::ivec2& BR, glm::vec2 top_left, glm::vec2 bottom_right, glm::ivec2 screen_size)
{
	TL = glm::ivec2(top_left.x * (float)ScreenSize.x, top_left.y * (float)ScreenSize.y);
	BR = glm::ivec2(bottom_right.x * (float)ScreenSize.x, bottom_right.y * (float)ScreenSize.y);
}

SDL_Surface* UIParent::LoadImageFromDisk(void** data, std::string path, bool silence)
{
	// Load image
	int w, h, c;
	*data = stbi_load(path.c_str(), &w, &h, &c, STBI_rgb);

	// Check if image loading
	if (*data == nullptr)
	{
		if (!silence)
			printf("stbi_load failed: %s\n", stbi_failure_reason());
		return nullptr;
	}

	// Create surface from image
	SDL_Surface* Surface = SDL_CreateRGBSurfaceFrom(*data, w, h, 8 * 3, w * 3, 0x000000FF, 0x0000FF00, 0x00FF0000, 0x00000000);

	// Check if surface creation
	if (Surface == nullptr)
	{
		printf("SDL_CreateRGBSurfaceFrom failed: %s\n", SDL_GetError());
		return nullptr;
	}

	return Surface;
}

SDL_Surface* UIParent::LoadImageFromURL(void** data, std::string URL, std::string Path, GottenBricks* GB_Item, Sets* S_Item)
{
	std::string ImagePath;
	if (!Path.empty())
		ImagePath = Path;
	else
	{
		// Load image not found if no URL given
		if (URL.empty())
			return LoadImageFromDisk(data, "Files/Images/NotFound.png", false);

		// Init variables
		std::string Type = URL.substr(URL.length() - 4);
		if ((GB_Item == nullptr && S_Item == nullptr) || (GB_Item != nullptr && S_Item != nullptr))
			abort();
		else if (GB_Item != nullptr)
		{
			// Part
			ImagePath = "Files/Images/Parts/" + GB_Item->PartNum + "_" + std::to_string(GB_Item->ColorID) + Type;
		}
		else
		{
			// Set
			ImagePath = "Files/Images/Sets/" + S_Item->SetNum + Type;
		}
	}

	SDL_Surface* Output = nullptr;

	// Load the image
	Output = LoadImageFromDisk(data, ImagePath, true);
	if (Output != nullptr)
		return Output;

	// Load NotFound if everything else failed
	return LoadImageFromDisk(data, "Files/Images/NotFound.png", false);
}

void UIParent::FreeImage(void* data)
{
	stbi_image_free(data);
}